/*************************************/
/* Lua engine for Hiew (c) FSC, 2007 */
/*************************************/

/********************************************************************************/
/* IMPORTANT: ALL EXPORTED LUA FUNCTIONS USE __stdcall CALLING CONVENTION (/Gz) */
/********************************************************************************/

#ifndef HIEWLUA_H
#define HIEWLUA_H

#include <windows.h>
#include "lua.hpp"

typedef lua_State *PLUASTATE;

typedef void (__stdcall *PPRINTTEXT)(LPCTSTR pText);
typedef void (__stdcall *PLUAREGISTRATION)(PLUASTATE L);

//
// Error codes
//
#define HIEWLUA_ERR_ALREADYOPEN     -1000   // repeated attempt to call HIEWLUA_OPEN was made
#define HIEWLUA_ERR_OPENFAILED      -1001   // lua_open() has failed
#define HIEWLUA_ERR_STILLRUNNING    -1002   // script is still running (returned from HIEWLUA_ISSCRIPTDONE call)
#define HIEWLUA_ERR_CLOSED          -1003   // can't execute scrip because HIEWLUA_OPEN call hasn's been issued
#define HIEWLUA_ERR_INTERNALERROR   -1004   // something went terribly wrong...
#define HIEWLUA_ERR_EXCEPTION       -1005   // an exception has occured at script compilation or execution
#define HIEWLUA_ERR_NOMEMORY        -1006   // there is not enough memory
#define HIEWLUA_ERR_FORCEDEXIT      -1007   // TerminateThread() has been called to stop the execution thread

//
// Plugin commands available to callers
//
#define HIEWLUA_OPEN                0       // open Lua
#define HIEWLUA_CLOSE               1       // close Lua
#define HIEWLUA_EXECSTRING          2       // execute script given as an asciiz string
#define HIEWLUA_EXECFILE            3       // execute script placed in a file
#define HIEWLUA_GETSCRIPTEXITCODE   4       // get exit code of script thread or HIEWLUA_ERR_STILLRUNNING if the thread has not finished yet
#define HIEWLUA_ABORTSCRIPT         5       // abort running script
#define HIEWLUA_SETSTDOUT           6       // set output routine
#define HIEWLUA_SETSTDERR           7       // set output routine for error messages
#define HIEWLUA_REGISTER            8       // set routine that performs additional Lua registration
#define HIEWLUA_VERSIONMAJOR        9       // get major version of the plugin
#define HIEWLUA_VERSIONMINOR        10      // get minor version of the plugin
#define HIEWLUA_TERMINATESCRIPT     11      // terminate script and wait until it is really terminated. Terminate parameter gives the timeout before the compulsory script (== thread) termination

//
// Structure to pass parameters for HIEWLUA_EXECFILE and HIEWLUA_EXECSCRIPT commands
// Please note that the string passed in pName parameter MUST NOT be freed while script is still running
//
typedef struct _EXEC_SCRIPT
{
    LPCTSTR pName;                      // filename for HIEWLUA_EXECFILE or text of the script to execute for HIEWLUA_EXECSCRIPT
    HANDLE hEvent;                      // handle of event to signal when the script execution is over or 0 if such signalling is not necessary
    HWND hWnd;                          // handle of window to send specified message (see later) when script execution is over or 0
    UINT Msg;                           // message to send if hWnd is !0
    WPARAM wParam;                      // message parameters
    LPARAM lParam;                      // same
} EXEC_SCRIPT, *PEXEC_SCRIPT;

//
// Main structure used to pass commands/parameters to HiewLua
//
typedef struct _HIEWLUA_COMMAND
{
    int cbSize;                         // size of this structure
    DWORD   Command;                    // command to execute
    union
    {
        EXEC_SCRIPT StringScript;       // HIEWLUA_EXECSTRING
        EXEC_SCRIPT FileScript;         // HIEWLUA_EXECFILE
        PPRINTTEXT pPrintText;          // HIEWLUA_SETSTDOUT, HIEWLUA_SETSTDERR
        PLUAREGISTRATION pRegister;     // HIEWLUA_REGISTER
        DWORD Timeout;                  // HIEWLUA_TERMINATESCRIPT
    };
} HIEWLUA_COMMAND, *PHIEWLUA_COMMAND;

#endif